@@ -3,14 +3,21 @@ package Module::Install;
 
 use 5.004;
 use strict 'vars';
+
 use vars qw{$VERSION};
 BEGIN {
-    # Don't forget to update Module::Install::Admin too!
-    $VERSION = '0.56';
+    # All Module::Install core packages now require synchronised versions.
+    # This will be used to ensure we don't accidentally load old or
+    # different versions of modules.
+    # This is not enforced yet, but will be some time in the next few
+    # releases once we can make sure it won't clash with custom
+    # Module::Install extensions.
+    $VERSION = '0.59';
 }
 
 # inc::Module::Install must be loaded first
-unless ( $INC{join('/', inc => split(/::/, __PACKAGE__)).'.pm'} ) {
+my $file = join( '/', 'inc', split /::/, __PACKAGE__ ) . '.pm';
+unless ( $INC{$file} ) {
     die <<"END_DIE";
 Please invoke ${\__PACKAGE__} with:
 
@@ -24,26 +31,25 @@ END_DIE
 }
 
 use Cwd        ();
-use FindBin;
 use File::Find ();
 use File::Path ();
+use FindBin;
 
 *inc::Module::Install::VERSION = *VERSION;
-@inc::Module::Install::ISA     = 'Module::Install';
+@inc::Module::Install::ISA     = __PACKAGE__;
 
 sub autoload {
-    my $self   = shift;
-    my $caller = $self->_caller;
-    my $cwd    = Cwd::cwd();
-    my $sym    = "$caller\::AUTOLOAD";
-
+    my $self = shift;
+    my $who  = $self->_caller;
+    my $cwd  = Cwd::cwd();
+    my $sym  = "${who}::AUTOLOAD";
     $sym->{$cwd} = sub {
         my $pwd = Cwd::cwd();
         if ( my $code = $sym->{$pwd} ) {
             # delegate back to parent dirs
             goto &$code unless $cwd eq $pwd;
         }
-        $$sym =~ /([^:]+)$/ or die "Cannot autoload $caller - $sym";
+        $$sym =~ /([^:]+)$/ or die "Cannot autoload $who - $sym";
         unshift @_, ($self, $1);
         goto &{$self->can('call')} unless uc($1) eq $1;
     };
@@ -52,18 +58,18 @@ sub autoload {
 sub import {
     my $class = shift;
     my $self  = $class->new(@_);
+    my $who   = $self->_caller;
 
     unless ( -f $self->{file} ) {
         require "$self->{path}/$self->{dispatch}.pm";
         File::Path::mkpath("$self->{prefix}/$self->{author}");
-        $self->{admin} = 
-          "$self->{name}::$self->{dispatch}"->new(_top => $self);
+        $self->{admin} = "$self->{name}::$self->{dispatch}"->new( _top => $self );
         $self->{admin}->init;
         @_ = ($class, _self => $self);
         goto &{"$self->{name}::import"};
     }
 
-    *{$self->_caller . "::AUTOLOAD"} = $self->autoload;
+    *{"${who}::AUTOLOAD"} = $self->autoload;
     $self->preload;
 
     # Unregister loader and worker packages so subdirs can use them again
@@ -74,11 +80,11 @@ sub import {
 sub preload {
     my ($self) = @_;
 
-        unless ( $self->{extentions} ) {
-                $self->load_extensions(
-                        "$self->{prefix}/$self->{path}", $self
-                        );
-        }
+    unless ( $self->{extensions} ) {
+        $self->load_extensions(
+            "$self->{prefix}/$self->{path}", $self
+        );
+    }
 
     my @exts = @{$self->{extensions}};
     unless ( @exts ) {
@@ -86,21 +92,21 @@ sub preload {
         @exts = $admin->load_all_extensions;
     }
 
-    my %seen_method;
+    my %seen;
     foreach my $obj ( @exts ) {
         while (my ($method, $glob) = each %{ref($obj) . '::'}) {
-            next unless defined *{$glob}{CODE};
+            next unless exists &{ref($obj).'::'.$method};
             next if $method =~ /^_/;
             next if $method eq uc($method);
-            $seen_method{$method}++;
+            $seen{$method}++;
         }
     }
 
-    my $caller = $self->_caller;
-    foreach my $name (sort keys %seen_method) {
-        *{"${caller}::$name"} = sub {
-            ${"${caller}::AUTOLOAD"} = "${caller}::$name";
-            goto &{"${caller}::AUTOLOAD"};
+    my $who = $self->_caller;
+    foreach my $name ( sort keys %seen ) {
+        *{"${who}::$name"} = sub {
+            ${"${who}::AUTOLOAD"} = "${who}::$name";
+            goto &{"${who}::AUTOLOAD"};
         };
     }
 }
@@ -118,30 +124,26 @@ sub new {
 
     $args{dispatch} ||= 'Admin';
     $args{prefix}   ||= 'inc';
-    $args{author}   ||= '.author';
+    $args{author}   ||= ($^O eq 'VMS' ? '_author' : '.author');
     $args{bundle}   ||= 'inc/BUNDLES';
     $args{base}     ||= $base_path;
-
     $class =~ s/^\Q$args{prefix}\E:://;
     $args{name}     ||= $class;
     $args{version}  ||= $class->VERSION;
-
-    unless ($args{path}) {
+    unless ( $args{path} ) {
         $args{path}  = $args{name};
         $args{path}  =~ s!::!/!g;
     }
     $args{file}     ||= "$args{base}/$args{prefix}/$args{path}.pm";
 
-    bless(\%args, $class);
+    bless \%args, $class;
 }
 
 sub call {
-    my $self   = shift;
-    my $method = shift;
-    my $obj    = $self->load($method) or return;
-
-    unshift @_, $obj;
-    goto &{$obj->can($method)};
+	my ($self, $method) = @_;
+	my $obj = $self->load($method) or return;
+        splice(@_, 0, 2, $obj);
+	goto &{$obj->can($method)};
 }
 
 sub load {
@@ -167,13 +169,14 @@ END_DIE
 }
 
 sub load_extensions {
-    my ($self, $path, $top_obj) = @_;
+    my ($self, $path, $top) = @_;
 
-    unshift @INC, $self->{prefix}
-        unless grep { $_ eq $self->{prefix} } @INC;
+    unless ( grep { lc $_ eq lc $self->{prefix} } @INC ) {
+        unshift @INC, $self->{prefix};
+    }
 
     local @INC = ($path, @INC);
-    foreach my $rv ($self->find_extensions($path)) {
+    foreach my $rv ( $self->find_extensions($path) ) {
         my ($file, $pkg) = @{$rv};
         next if $self->{pathnames}{$pkg};
 
@@ -184,7 +187,7 @@ sub load_extensions {
             next;
         }
         $self->{pathnames}{$pkg} = delete $INC{$file};
-        push @{$self->{extensions}}, &{$new}($pkg, _top => $top_obj );
+        push @{$self->{extensions}}, &{$new}($pkg, _top => $top );
     }
 
     $self->{extensions} ||= [];
@@ -197,10 +200,32 @@ sub find_extensions {
     File::Find::find( sub {
         my $file = $File::Find::name;
         return unless $file =~ m!^\Q$path\E/(.+)\.pm\Z!is;
-        return if $1 eq $self->{dispatch};
+        my $subpath = $1;
+        return if lc($subpath) eq lc($self->{dispatch});
+
+        $file = "$self->{path}/$subpath.pm";
+        my $pkg = "$self->{name}::$subpath";
+        $pkg =~ s!/!::!g;
+
+        # If we have a mixed-case package name, assume case has been preserved
+        # correctly.  Otherwise, root through the file to locate the case-preserved
+        # version of the package name.
+        if ( $subpath eq lc($subpath) || $subpath eq uc($subpath) ) {
+            open PKGFILE, "<$subpath.pm" or die "find_extensions: Can't open $subpath.pm: $!";
+            my $in_pod = 0;
+            while ( <PKGFILE> ) {
+                $in_pod = 1 if /^=\w/;
+                $in_pod = 0 if /^=cut/;
+                next if ($in_pod || /^=cut/);  # skip pod text
+                next if /^\s*#/;               # and comments
+                if ( m/^\s*package\s+($pkg)\s*;/i ) {
+                    $pkg = $1;
+                    last;
+                }
+            }
+            close PKGFILE;
+        }
 
-        $file = "$self->{path}/$1.pm";
-        my $pkg = "$self->{name}::$1"; $pkg =~ s!/!::!g;
         push @found, [ $file, $pkg ];
     }, $path ) if -d $path;
 
@@ -208,15 +233,13 @@ sub find_extensions {
 }
 
 sub _caller {
-    my $depth  = 0;
-    my $caller = caller($depth);
-
-    while ($caller eq __PACKAGE__) {
+    my $depth = 0;
+    my $call  = caller($depth);
+    while ( $call eq __PACKAGE__ ) {
         $depth++;
-        $caller = caller($depth);
+        $call = caller($depth);
     }
-
-    $caller;
+    return $call;
 }
 
 1;
